Package: MRgrowth
Type: Package
Title: Mark-Recapture Growth Models
Version: 0.1.2
Authors@R: person(
    given = c("Donald", "T."),
    family = "McKnight",
    role = c("aut", "cre"),
    email = "donald.mcknight@my.jcu.edu.au",
    comment = c(affiliation = "Savanna Field Station"))
Date: 2026-04-06
Maintainer: Donald T. McKnight <donald.mcknight@my.jcu.edu.au>
Description: Researchers often need to calculate body-size growth rates for individuals
    that do not have associated age data. These growth rates are based on
    mark-recapture data where an individual was captured and measured at time 1
    then recaptured and measured at time 2. The sizes at each time and amount
    of time between captures can be used to calculate growth rates. 'MRgrowth'
    follows the approach in Edmonds et al. (2021) <doi:10.1371/journal.pone.0259978>
    and provides functions to calculate growth using three formulas, the
    Faben's reformulation of the von Bertalanffy formula, the Gompertz 
    formula, and a logistic formula.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Suggests: knitr,rmarkdown
Depends: R (>= 3.5)
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-06 23:11:29 UTC; Donald McKnight
Author: Donald T. McKnight [aut, cre] (affiliation: Savanna Field Station)
Repository: CRAN
Date/Publication: 2026-04-10 14:20:07 UTC
Built: R 4.5.2; ; 2026-04-15 04:55:52 UTC; unix

export(age.at.size)
export(size.at.age)
export(calc.growth)
export(format.data)
export(logistic)
export(gompertz)
export(vb)
importFrom("grDevices", "adjustcolor")
importFrom("graphics", "legend", "lines", "polygon")
importFrom("stats", "dnorm", "median", "na.omit", "optim", "quantile", "setNames")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----data---------------------------------------------------------------------

library(MRgrowth)

data(MRdata)
head(MRdata)
str(MRdata)

## ----format-------------------------------------------------------------------
formatted.data <- format.data(
  x        = MRdata,
  id.col   = "id",
  date.col = "date",
  size.col = "size",
  units    = "years",
  retain   = "max"
)

head(formatted.data)

## ----fit_fast-----------------------------------------------------------------
# Using low runs for vignette build speed
results <- calc.growth(
  x           = formatted.data,
  size0       = 50,
  age.vect    = 1:50,
  models      = c("vb", "gompertz", "logistic"),
  type        = "ML",
  runs        = 100,
  return.plot = TRUE
)

## ----params-------------------------------------------------------------------
results$parameters

## ----sizes--------------------------------------------------------------------
head(results$sizes.at.ages)

## -----------------------------------------------------------------------------
#look at model diagnostics
print(results$parameters)


# Extract parameters from the best-fitting model
best.model <- results$parameters[which(results$parameters$model=="vb"), ]


## ----size_at_age--------------------------------------------------------------
#calculate expected sizes for a 10, 20 and 30 year old turtle
#note that the model argument specifies that we are using a vb model

size.at.age(
  A     = best.model$A,
  k     = best.model$k,
  age   = c(10, 20, 30),
  size0 = 50,
  model = "vb")


## ----age_at_size--------------------------------------------------------------

#Estimate the age (years) of a 150, 250, and 350 mm turtle
#note that the model argument specifies that we are using a vb model

age.at.size(
  A     = best.model$A,
  k     = best.model$k,
  size  = c(150, 250, 350),
  size0 = 50,
  model = "vb")


## ----growth_funcs-------------------------------------------------------------
# Predict size of a 200 mm turtle after 3 years using the vb model
#note that unlike most functions in 'MRgrowth', each model as a separate function, rather than specifying the model in a single function

vb(
  A     = best.model$A,
  k     = best.model$k,
  size1 = 200,
  td    = 3
)

## -----------------------------------------------------------------------------

predicted.size2 <- vb(
  A     = best.model$A,
  k     = best.model$k,
  size1 = formatted.data$size1, #use size 1 data
  td    = formatted.data$td #use time difference in the data
)

plot(x = formatted.data$size2,
     y = predicted.size2,
     xlab = "Observed size2",
     ylab = "Estimated size2",
     pch = 16,
     col = "blue",
     bg  = "white",
     panel.first = rect(par("usr")[1], par("usr")[3],
                        par("usr")[2], par("usr")[4],
                        col = "white"))
abline(lm(predicted.size2 ~ formatted.data$size2),
       col = "black",
       lwd = 1)
r2 <- summary(lm(predicted.size2 ~ formatted.data$size2))$r.squared
legend("topleft",
       legend = bquote(R^2 == .(round(r2, 4))),
       bty    = "n")


## -----------------------------------------------------------------------------
#remove any individuals where size1 > A
observed.size2.b <- formatted.data$size2[which(formatted.data$size1 < best.model$A)]
predicted.size2.b <- predicted.size2[which(formatted.data$size1 < best.model$A)]

plot(x = observed.size2.b,
     y = predicted.size2.b,
     xlab = "Observed size2",
     ylab = "Estimated size2",
     pch = 16,
     col = "blue",
     bg  = "white",
     panel.first = rect(par("usr")[1], par("usr")[3],
                        par("usr")[2], par("usr")[4],
                        col = "white"))
abline(lm(predicted.size2.b ~ observed.size2.b),
       col = "black",
       lwd = 1)
r2 <- summary(lm(predicted.size2.b ~ observed.size2.b))$r.squared
legend("topleft",
       legend = bquote(R^2 == .(round(r2, 4))),
       bty    = "n")


## -----------------------------------------------------------------------------
predicted.size <- size.at.age(A=best.model$A,
                              k=best.model$k,
                              age=c(5,10,12,20),
                              size0=50,
                              model="vb")

#expected sizes given their ages
print(predicted.size)

#percent difference between observed and predicted sizes
print((c(200,320,330,400)-predicted.size)/c(200,320,330,400)*100)

## -----------------------------------------------------------------------------
logistic.model <- results$parameters[which(results$parameters$model=="logistic"), ]

predicted.size.logistic <- size.at.age(A=logistic.model$A,
                              k=logistic.model$k,
                              age=c(5,10,12,20),
                              size0=50,
                              model="logistic")

#predicted sizes based on the logistic model
print(predicted.size.logistic)

#percent difference between observed and predicted sizes for the logistic model
print((c(200,320,330,400)-predicted.size.logistic)/c(200,320,330,400)*100)


## -----------------------------------------------------------------------------
predicted.age <- age.at.size(A=best.model$A,
                              k=best.model$k,
                              size=c(200,320,330,400),
                              size0=50,
                              model="vb")

print(predicted.age)

#' Calculate growth curves
#'
#' Calculate, compare, and plot growth curves based on different models
#'
#' @param x (data frame) Output from \code{MRgrowth::format.data()} Data frame with columns of size at capture, size at recapture, and amount of time between captures.
#' @param size0 (numeric) Initial body size at birth/hatching (should be in the same units used to calculate A and k). This will generally be a species average, rather than being specific per individual, but it can be a vector of specific birth/hatching sizes per individual if known. Note that this will not affect the final values of A, k, or their confidence intervals, but it will affect the estimated sizes at ages.
#' @param age.vect (numeric vector) Vector of ages for which you would like expected sizes and confidence intervals around the size (used for plotting growth curves). If age.vect is not supplied, a vector from 1 to 4x the largest time interval will be used (default = NULL)
#' @param runs (integer) The number of bootstraps to use for calculating confidence intervals. Large values result in long run times (see Vignette; default = 5000)
#' @param models (character vector) The model(s) used to calculate A and k. Can be any combination of "vb", "gompertz","logistic". Fitted parameters, model diagnostics, and sizes at expected ages will be returned for all included models (default = c("vb", "gompertz","logistic"))
#' @param fixed.A (numeric) This fixes the asymptotic size (A) at the specified value, meaning that only the growth rate (k) will be estimated. Use with caution (see vignette; default = NULL)
#' @param initial.A (numeric) Starting value for the search algorithm for the asymptotic size (A). This has no impact on the results, but an initial.A that is close to the final A will result in a faster run time (default = max size in your data set).
#' @param initial.k (numeric) Starting value for the search algorithm for the growth rate (k). This has no impact on the results, but an initial.k that is close to the final k will result in a faster run time (default = 0.5).
#' @param size1.col (character) Name of column containing size data at capture (default = "size1"; the output from \code{MRgrowth::format.data()})
#' @param size2.col (character) Name of column containing size data at recapture (default = "size2"; the output from \code{MRgrowth::format.data()})
#' @param times.col (character) Name of column containing time interval between captures (default = "td"; the output from \code{MRgrowth::format.data()})
#' @param type (character) Method used of fitting the model. One of "ML" (Maximum Likelihood) or "OLS" (Ordinary Least Squares; default = "ML")
#' @param seed (integer) Seed to use for random selections when calculating bootstrapped confidence intervals (default = 1701).
#' @param return.plot (logical) Determines whether a plot of growth curves should be printed (default = TRUE)
#' @param optimizer (character) One of the optimization methods available in \code{\link[stats]{optim}} (default = "L-BFGS-B")
#'
#' @return A list of two objects:
#' \describe{
#'  \item{parameters}{A data frame with one row per model containing the following columns:
#'     \describe{
#'       \item{model}{The growth model fitted}
#'       \item{A}{Estimated asymptotic size}
#'       \item{A.lower}{Lower 95\% bootstrap confidence interval for A}
#'       \item{A.upper}{Upper 95\% bootstrap confidence interval for A}
#'       \item{k}{Estimated growth coefficient}
#'       \item{k.lower}{Lower 95\% bootstrap confidence interval for k}
#'       \item{k.upper}{Upper 95\% bootstrap confidence interval for k}
#'       \item{convergence}{Convergence code from \code{\link[stats]{optim}}; 0 indicates successful convergence}
#'       \item{NLL}{Negative log-likelihood of the fitted model (only returned when \code{type="ML"})}
#'       \item{SS}{Sum of squares (only returned when \code{type="OLS"})}
#'       \item{AIC}{Akaike Information Criterion (only returned when \code{type="ML"})}
#'       \item{AICc}{Sample size corrected AIC (only returned when \code{type="ML"})}
#'       \item{fixed.AIC}{Akaike Information Criterion calculated with 2 parameters when A is fixed (only returned when \code{type="ML"} and a value is entered for \code{fixed.A})}
#'       \item{fixed.AICc}{Sample size corrected AIC calculated with 2 parameters when A is fixed (only returned when \code{type="ML"} and a value is entered for \code{fixed.A})}
#'
#'     }
#'   }
#'   \item{sizes.at.ages}{A data frame with one row per age per model containing the following columns:
#'     \describe{
#'       \item{model}{The growth model fitted}
#'       \item{age}{Age from \code{age.vect}}
#'       \item{size}{Predicted size at a given age}
#'       \item{upper}{Upper 95\% bootstrap confidence interval for predicted size}
#'       \item{lower}{Lower 95\% bootstrap confidence interval for predicted size}
#'     }
#'   }
#' }
#'
#' @details
#' This is the primary function for MRgrowth. It takes the output from \code{format.data()}, calculates growth curves the selected model(s), and returns the model parameters and the data for plotting curves. The output includes model diagnostics like AIC values that can be used to evaluate model fit and compare models.
#'
#' @examples
#'
#' data(MRdata)
#'
#' #format the data
#' formatted.data <- format.data(
#'  x = MRdata,
#'  id.col = "id",
#'  date.col = "date",
#'  size.col = "size",
#'  units = "years",
#'  retain = "max")
#'
#' #get growth curves for the gompertz and logistic methods.
#' #runs are set to 10 to keep the run time low.
#' #For actual CI estimates, much higher values are needed
#' #likewise, the plot function has been turned off
#' g.result <- calc.growth(formatted.data,
#'    size0=50,
#'    age.vect=c(1:40),
#'    models=c("gompertz","logistic"),
#'    runs=10,
#'    return.plot=FALSE)
#'
#' @references
#' von Bertalanffy, L. (1957). Quantitative laws in metabolism and growth.
#' \emph{The Quarterly Review of Biology}, 32, 217-231.
#'
#' Fabens, A.J. (1965). Properties and fitting of the Von Bertalanffy growth curve.
#' \emph{Growth}, 29, 265-289.
#'
#' Gompertz, B. (1833). On the nature of the function expressive of the law of human
#' mortality, and on a new mode of determining the value of life contingencies.
#' \emph{Philosophical Transactions of the Royal Society of London}, 123, 252-253.
#'
#' Schoener, T.W. and Schoener, A. (1978). Estimating and interpreting body-size
#' growth in some Anolis lizards. \emph{Copeia}, 1978, 390-405.
#'
#' @export
calc.growth <- function(x,size0=NULL,age.vect=NULL,models = c("vb","gompertz","logistic"),fixed.A=NULL,initial.A=NULL,initial.k=0.5,size1.col="size1",size2.col="size2",times.col="td",type="ML",runs=5000,seed=1701,return.plot=TRUE,optimizer="L-BFGS-B"){

  if(!all(models %in% c("vb", "gompertz", "logistic"))){
    stop("Invalid model name(s). The models argument must only contain only 'vb', 'gompertz', and/or 'logistic'.", call. = FALSE)}

  if(length(type) != 1 || !type %in% c("OLS", "ML")){
    stop("Invalid type. The type argument must contain one of 'OLS' or 'ML'.", call. = FALSE)}

  #set lower bounds for models based on input settings
  if (is.null(fixed.A)) {
    ssq.lower <- c(1e-6, 1e-6)
    ml.lower  <- c(1e-6, 1e-6, 1e-6)
  } else {
    ssq.lower <- c(1e-6)
    ml.lower  <- c(1e-6, 1e-6)
  }

  #rename columns
  colnames(x)[which(colnames(x)==size1.col)] <- "size1"
  colnames(x)[which(colnames(x)==size2.col)] <- "size2"
  colnames(x)[which(colnames(x)==times.col)] <- "times"

  #subset to only necessary columns and remove NAs
  x <- x[,c("size1","size2","times")]
  x <- na.omit(x)

  #extract as vectors for later functions
  size1 <- x[,"size1"]
  size2 <- x[,"size2"]
  times <- x[,"times"]


  if(is.null(age.vect)==TRUE){
    warning("No values were provided for age.vect. This will not affect the estimation of A, k, or their associated CIs, but predicted sizes will be calculated and plotted for age 1 to 4 times the maximum time difference in your data set. This may not cover the full age range of your species. Update the age.vect argument accordingly.")
    age.vect <- 1:round((max(times)*4))}

  if(is.null(size0)==TRUE){
    warning("No value was provided for size0. This will not affect the estimation of A, k, or their associated CIs, but will affect the predicted sizes at each age in age.vect and the associated plot. The smallest size in your data divided by 2 has been used as size0. This may give highly misleading predicted sizes. Please update with an appropriate value for your species")
    size0 <- min(size1)/2}

  if(is.null(initial.A)==TRUE){initial.A <-max(size2,na.rm=TRUE)}

  start.vals <- c(A=initial.A,k=initial.k) #make vector of start values

  #if(is.null(fixed.A)==FALSE){start.vals <- start.vals[2]} #changed.here

  sizes.a <- vector("list",length(models)) #make empty list
  params.a <- vector("list",length(models)) #make empty list
  for(a in 1:length(models)){

    model <- models[a]

    #calculate with least squares first to get good starting values for ML
    #changed here
    fit1 <- optim(if(is.null(fixed.A)==FALSE){start.vals[2]}else{start.vals}, ssq , size1=size1, size2=size2, td=times,method = optimizer,lower = ssq.lower,model=model,fixed.A=fixed.A)

    converg <- fit1$convergence

    if(is.null(fixed.A)==TRUE){
      fit <- fit1$par}else{ #get parameters
      fit <- fit1$par
      fit[2] <- fit[1]
      fit[1] <- fixed.A}

    if(type == "ML"){

      if(is.null(fixed.A)==TRUE){
        start.vals.ml <- c(fit[1],fit[2],sigma=sqrt(fit1$value/nrow(x)))}else{ #update starting values based on ssq
          start.vals.ml <- c(fit[2],sigma=sqrt(fit1$value/nrow(x)))}

      #get ML estimate
      fit.ml <- optim(start.vals.ml,ml,model=model,size1= size1,td= times, size2 = size2,method = optimizer,  lower  = ml.lower,fixed.A=fixed.A  )

      NLL <- fit.ml$value #negative log likelihood
      converg <- fit.ml$convergence

      if(is.null(fixed.A)==TRUE){
          fit <- fit.ml$par

          #get AIC and AICc values
          AIC  <- 2 * NLL + 2 * 3
          AICc <- AIC + (2 * 3 * (3 + 1)) / (nrow(x) - 3 - 1)}else

            {
            fit <- fit1$par
            fit[2] <- fit[1]
            fit[1] <- fixed.A
            fit[[3]] <- fit1$par[2]
            names(fit)[3] <- "sigma"

            #get AIC and AICc values
            AIC  <- 2 * NLL + 2 * 3 #calculates with 3 parameters even though only 2 were tested
            AICc <- AIC + (2 * 3 * (3 + 1)) / (nrow(x) - 3 - 1) #calculates with 3 parameters even though only 2 were tested
            fixed.AIC  <- 2 * NLL + 2 * 2 #calculates with 2 parameters for the fixed AIC
            fixed.AICc <- fixed.AIC + (2 * 2 * (2 + 1)) / (nrow(x) - 2 - 1)}
    }

    #calculate size at each age for the vector of ages provided
    sizes.at.ages <- size.at.age(fit[1],fit[2],age=age.vect,size0=size0,model=model)

  #get confidence intervals
    CI <- if(type == "ML"){
      bootstrap.ml(nboot = runs, size1=size1, size2=size2, td=times, start.vals=start.vals, ages=age.vect, size0=size0,model=model,seed=seed,fixed.A=fixed.A,optimizer=optimizer)}else{
        bootstrap.ssq(nboot = runs, size1=size1, size2=size2, td=times, start.vals=start.vals, ages=age.vect, size0=size0,model=model,seed=seed,fixed.A=fixed.A,optimizer=optimizer)}

    param <- CI$params.ci[1:2,1:4] #remove unneeded columns and rows
    param[1:2,"est"] <- fit[1:2] #replace the bootstrap fitted values with the final fitted values

    param <- rbind.data.frame(c(param[1,2:4],param[2,2:4]))
    colnames(param) <- c("A","A.lower","A.upper","k","k.lower","k.upper")
    param$convergence <- converg
    if(type=="ML"){
      param$NLL <- NLL
      param$AIC <- AIC
      param$AICc <- AICc
      if(is.null(fixed.A)==FALSE){
        param$fixed.AIC <- fixed.AIC
        param$fixed.AICc <- fixed.AICc      }
    }else{
        param$SS <- fit1$value
      }
    params.a[[a]] <- cbind.data.frame(model=model,param)

    sizes.a[[a]] <- cbind.data.frame(model=model,age = CI$size.ci$ages,size=sizes.at.ages,upper=CI$size.ci$upper,lower=CI$size.ci$lower)

  } #close for a in 1:length(models)

  params.a <- do.call("rbind.data.frame",params.a)

  sizes.a <- do.call("rbind.data.frame",sizes.a)

  if(return.plot==TRUE){
    models.p <- unique(sizes.a$model)
    cols   <- setNames(c("#0472C1","#FFC928","#6D6D6D")[1:length(models.p)],
                       models.p)  # extend if more than 3 models

    plot(NULL,
         xlim = range(sizes.a$age),
         ylim = range(c(sizes.a$upper, sizes.a$lower)),
         xlab = "Age",
         ylab = "Size",
         las  = 1)

    for (m in models.p) {
      sub <- sizes.a[sizes.a$model == m, ]
      sub <- sub[order(sub$age), ]   # ensure lines connect in order

      # Shaded CI ribbon — no border
      polygon(x      = c(sub$age, rev(sub$age)),
              y      = c(sub$upper, rev(sub$lower)),
              col    = adjustcolor(cols[m], alpha.f = 0.2),
              border = NA)

      # Central line
      lines(sub$age, sub$size,
            col = cols[m],
            lwd = 3)
    }

    legend("bottomright",
           legend = models.p,
           col    = cols[models.p],
           lwd    = 3,
           bty    = "n")}


  list(parameters = params.a,sizes.at.ages = sizes.a)
}

#' Logistic growth
#'
#' Calculate size after a given time interval using the logistic formula
#'
#' @param A (numeric) Asymptotic body size (can be calculated in \code{calc.growth()})
#' @param k (numeric) Growth rate (can be calculated in \code{calc.growth()})
#' @param size1 (numeric vector) Initial body size (should be in the same units used to calculate A and k)
#' @param td (numeric vector) Time since \code{size1} (should be in the same units used to calculate A and k)
#' @return Vector of expected size(s) after the time interval(s) specified in \code{td}
#'
#' @details
#' It takes a vector of initial sizes, vector of amounts of time since those sizes were measured, and A and k from \code{calc.growth(model="logistic")} and returns the expected body sizes after the specified time intervals.
#'
#' \strong{Note}: If size1 > A, the resulting size estimate will be less than size1
#'
#' @examples
#' #For this example, assume size is in mm and time is in years
#' #calculate expected sizes of individuals that are 200 and 300 mm after 2 and 1.5 years, respectively
#' logistic(A = 393.0270,
#'  k = 0.2835795,
#'  size1 = c(200,300),
#'  td = c(2,1.5))
#'
#' @references
#' Schoener, T.W. and Schoener, A. (1978). Estimating and interpreting body-size
#' growth in some Anolis lizards. \emph{Copeia}, 1978, 390-405.
#'
#' @export
logistic<- function(A,k, size1,td){
  size2 <- (A*size1)/(size1+(A-size1)*exp(-k*td))
  size2}

#' Example mark-recapture data set
#'
#' Example mark-recapture data set of male and female turtles from two populations
#'
#' @format A data frame with 43 rows and 3 variables:
#' \describe{
#'   \item{id}{Unique ID for each individual}
#'   \item{size}{Individual body size at each capture}
#'   \item{date}{Date of each measrement in Date format}
#' }
#'
#' @source Hypothetical data over 2-3 capture events
#'
#' @usage data(MRdata)
#'
"MRdata"

#' von Bertalanffy growth
#'
#' Calculate size after a given time interval using the Faben's reformulation of the von Bertalanffy formula
#'
#' @param A (numeric) Asymptotic body size (can be calculated in \code{calc.growth()})
#' @param k (numeric) Growth rate (can be calculated in \code{calc.growth()})
#' @param size1 (numeric vector) Initial body size (should be in the same units used to calculate A and k)
#' @param td (numeric vector) Time since \code{size1} (should be in the same units used to calculate A and k)
#' @return Vector of expected size(s) after the time interval(s) specified in \code{td}
#'
#' @details
#' It takes a vector of initial sizes, vector of amounts of time since those sizes were measured, and A and k from \code{calc.growth(model="vb")} and returns the expected body sizes after the specified time intervals.
#'
#' \strong{Note}: If size1 > A, the resulting size estimate will be less than size1
#'
#' @examples
#' #For this example, assume size is in mm and time is in years
#' #calculate expected sizes of individuals that are 200 and 300 mm after 2 and 1.5 years, respectively
#' vb(A = 403.7077,
#'  k = 0.1365507,
#'  size1 = c(200,300),
#'  td = c(2,1.5))
#'
#' @references
#' von Bertalanffy, L. (1957). Quantitative laws in metabolism and growth.
#' \emph{The Quarterly Review of Biology}, 32, 217-231.
#'
#' Fabens, A.J. (1965). Properties and fitting of the von Bertalanffy growth curve.
#' \emph{Growth}, 29, 265-289.
#'
#' @export
vb<- function(A,k, size1,td){
  size2 <- A-(A-size1)*exp(-k*td)
  size2}

#' @noRd
ssq <- function(par, size1,td,size2,model,fixed.A=NULL) {
  if (is.null(fixed.A)) {
    A <- par[1]
    k <- par[2]
  } else {
    A <- fixed.A      # use the fixed value
    k <- par[1]       # only k is estimated
  }
  if(model=="gompertz"){pred <- gompertz(A=A,k=k,size1=size1,td=td)}else{
    if(model=="logistic"){pred <- logistic(A=A,k=k,size1=size1,td=td)}else{
      pred <- vb(A=A,k=k,size1=size1,td=td)
    }
  }
  sum((size2 - pred)^2)
}

#' @noRd
ml <- function(par, size1, td, size2,model,fixed.A) {
  if (is.null(fixed.A)) {
    A     <- par[1]
    k     <- par[2]
    sigma <- par[3]
  } else {
    A     <- fixed.A  # use the fixed value
    k     <- par[1]   # only k and sigma estimated
    sigma <- par[2]
  }

  if(model=="gompertz"){pred <- gompertz(A = A, k = k, size1 = size1, td = td)}else{
    if(model=="logistic"){pred <- logistic(A = A, k = k, size1 = size1, td = td)}else{
      pred <- vb(A = A, k = k, size1 = size1, td = td)}
      }

  # Sum of negative log-likelihood under normal distribution
  nll <- -sum(dnorm(size2, mean = pred, sd = sigma, log = TRUE))
  nll
}

#' Age at size
#'
#' Calculate expected age at a given size
#'
#' @param A (numeric) Asymptotic body size (can be calculated in \code{calc.growth()})
#' @param k (numeric) Growth rate (can be calculated in \code{calc.growth()})
#' @param size (numeric vector) The size(s) for which you want to know the age (should be in the same units used to calculate A and k)
#' @param size0 (numeric) Initial body size at birth/hatching (should be in the same units used to calculate A and k). This will generally be a species average, rather than being specific per individual, but it can be a vector of specific birth/hatching sizes per individual if known.
#' @param model (character) The model used to calculate A and k. One of "vb", "gompertz","logistic".
#' @return Vector of expected age(s)
#'
#' @details
#' It takes a vector of sizes, initial size for the species, and A and k from \code{calc.growth()} and returns the expected age at a given size based on the specified model.
#'
#' \strong{Note}: Age cannot be calculated for sizes >A, resulting in NaN.
#'
#' @examples
#' #For this example, assume size is in mm and time is in years
#' #Use the gompertz formula to calculate the expected age of a 350 mm
#' #turtle for a species that hatches at an average size of 50 mm
#' age.at.size(A = 396.6738,
#'  k = 0.2029850,
#'  size = 350,
#'  size0 = 50,
#'  model="gompertz")
#'
#' @export
age.at.size <- function(A, k, size, size0,model){
  if(model=="gompertz"){age <- -(1/k) * log( log(size/A) / log(size0/A))}else{
    if(model=="logistic"){age <- -(1/k) * log( (size0 * (A - size)) / (size * (A - size0)) )}else{
      age <- -(1/k) * log( (A - size) / (A - size0) )
    }
  }
  age}

#' Size at age
#'
#' Calculate expected size at a given age
#'
#' @param A (numeric) Asymptotic body size (can be calculated in \code{calc.growth()})
#' @param k (numeric) Growth rate (can be calculated in \code{calc.growth()})
#' @param age (numeric vector) The age(s) for which you want to know the size (should be in the same units used to calculate A and k)
#' @param size0 (numeric) Initial body size at birth/hatching (should be in the same units used to calculate A and k). This will generally be a species average, rather than being specific per individual, but it can be a vector of specific birth/hatching sizes per individual if known.
#' @param model (character) The model used to calculate A and k. One of "vb", "gompertz","logistic".
#' @return Vector of expected size(s)
#'
#' @details
#' It takes a vector of ages, initial size for the species, and A and k from \code{calc.growth()} and returns the expected body sizes at a given age based on the specified model.
#'
#' @examples
#' #For this example, assume size is in mm and time is in years
#' #Use the gompertz formula to calculate the expected size of a 5-year old
#' #turtle for a species that hatches at an average size of 50 mm
#' size.at.age(A = 396.6738,
#'  k = 0.2029850,
#'  age = 5,
#'  size0 = 50,
#'  model="gompertz")
#'
#' @export
size.at.age <- function(A, k, age, size0,model){
  if(model=="gompertz"){size <- A*(size0/A)^exp(-k*age)}else{
    if(model=="logistic"){size <- (A * size0) / (size0 + (A - size0) * exp(-k * age))}else{
      size <- A-(A-size0)*exp(-k*age)
    }
  }
  size
}

#' Format data
#'
#' Format data for other MRgrowth functions
#'
#' @param x (data frame) Input data containing one row per capture event and columns for individual ID, date/time (formatted as a Date or POSIX), and body size measurement.
#' @param id.col (character) Name of column containing unique individual IDs (default = "id")
#' @param date.col (character) Name of column containing dates/times (default = "date")
#' @param size.col (character) Name of column containing body size measurements (default = "size")
#' @param units (character) Units to be returned for time intervals between measurements. One of "hours", "days", "weeks", "years" (default = "years")
#' @param retain (character) Determines how the function should handle more than two measurements per individual. "max" returns the first and last measurement (after removing NAs), "random" = randomly selects two measurements (after removing NAs), "all" = keeps all measurements with growth intervals between measurements 1 and 2, 2 and 3, 3 and 4, etc. Note that "all" results in pseudoreplication. (default = "max")
#' @param seed (integer) Seed to use for random selections if \code{retain="random"} (default = 1701).
#'
#' @return Data frame of measurements and capture intervals formatted for MRgrowth functions. Each row is a pair of capture events and associated measurements for an individual.
#'
#' Column \code{id} = IDs as entered in the input data
#'
#' Column \code{size1} = size at capture
#'
#' Column \code{size2} = size at recapture
#'
#' Column \code{date1} = date at capture
#'
#' Column \code{date2} = date at recapture
#'
#' Column \code{td} = time difference between date1 and date2 in the units specified by the units argument.
#'
#' @details
#' It takes a long-format data frame of capture events and measurements (one row per individual per capture) and converts it into the wide-format (one row per pair of capture events per individual) needed for MRgrowth
#'
#' If \code{retain} is set to "random", only 1 pair of capture events will be retained per individual. The \code{seed} argument ensures repeatability of the code.
#'
#' \strong{Note}: Any NA data are removed as part of the function before applying the \code{retain} argument. Thus, if an individual was captured 3 times and only measured the 2nd and 3rd time, the data for the 2nd and 3rd capture events will be returned even if \code{retain="max"}
#'
#' \strong{Note}: Only individuals with at least 2 capture events (with no NAs) are returned. Any individuals that were never recaptured or did not have all data recorded for at least two recaptures are removed.
#'
#' @examples
#'
#' data(MRdata)
#'
#' #format data with years as the time units and the first and last
#' #measurement when more than 2 captures are present for an individual.
#' formatted.data <- format.data(
#'   x = MRdata,
#'   id.col = "id",
#'   date.col = "date",
#'   size.col = "size",
#'   units = "years",
#'   retain = "max"
#' )
#' @export

format.data <- function(x,id.col="id",date.col="date",size.col="size", units="years",retain="max",seed=1701){

  if(length(units) != 1 || !units %in% c("hours", "days", "weeks", "years" )){
    stop("Invalid unit. The units argument must contain one of 'hours', 'days','weeks','years'.", call. = FALSE)}

  if(length(retain) != 1 || !retain %in% c("max","random","all")){
    stop("Invalid retain. The retain argument must contain one of 'max', 'random','all'.", call. = FALSE)}


  set.seed(seed)

  data <- x

  #rename columns
  colnames(data)[which(colnames(data)==id.col)] <- "id"
  colnames(data)[which(colnames(data)==date.col)] <- "date"
  colnames(data)[which(colnames(data)==size.col)] <- "size"


  if(inherits(data$date,c("Date","POSIXt"))==FALSE){stop("date.col is not in a Date or POSIX format",call.=FALSE)}

  #subset to the columns needed for the function
  data <- data[,which(colnames(data) %in% c("id","date","size"))]

  #remove NAs
  data <- na.omit(data)

  #sort by id and date
  data <- data[with(data,order(id,date)),]

  gdata <- vector("list",length(unique(data$id)))
  for(i in 1:length(unique(data$id))){

    data.i <- data[which(data$id == unique(data$id)[i]),]

    if(nrow(data.i) > 1){ #only proceed if there is a second growth measure
      if(retain=="max"){data.i <- data.i[c(1,nrow(data.i)),]} #if retain is set to max, extract first and last data point
      if(retain=="random"){data.i <- data.i[sort(sample(1:nrow(data.i),2,replace=FALSE)),]} #if retain is set to random, randomly select two rows

      size1 <- data.i$size[1:(nrow(data.i)-1)] #extract initial sizes
      size2 <- data.i$size[2:nrow(data.i)] #extract recapture sizes
      date1 <- data.i$date[1:(nrow(data.i)-1)] #extract initial dates
      date2 <- data.i$date[2:nrow(data.i)] #extract recapture dates
      td <- as.numeric(difftime(date2,date1,units="days"))
      if(units=="hours"){td <- td*24}
      if(units=="weeks"){td <- td/7}
      if(units=="years"){td <- td/365}

      gdata[[i]] <- cbind.data.frame(id=data.i$id[1],size1,size2,date1,date2,td)
    } #close if(nrow(data.i) > 1)
  }#close for i in 1:length
  gdata <- do.call("rbind.data.frame",gdata)

  gdata
} #close format.data function

#' @noRd
bootstrap.ssq <- function(nboot, size1, size2, td, start.vals, ages, size0, model, seed=1701,fixed.A,optimizer="L-BFGS-B") {
  set.seed(seed)
  n <- length(size1)

  if (is.null(fixed.A)) {
    ssq.lower <- c(1e-6, 1e-6)}else{
    ssq.lower <- c(1e-6)}

  boot_params <- matrix(NA, nrow = nboot, ncol = 2,
                        dimnames = list(NULL, c("A", "k")))
  boot_preds <- matrix(NA, nrow = nboot, ncol = length(ages))

  for (b in 1:nboot) {
    idx <- sample(1:n, replace = TRUE)
    size1_b <- size1[idx]; size2_b <- size2[idx]; td_b <- td[idx]

    #changed.here
    fit_b <- try(
      optim(if(is.null(fixed.A)==FALSE){start.vals[2]}else{start.vals}, ssq, size1 = size1_b, size2 = size2_b, td = td_b,method = optimizer,lower = ssq.lower,model=model,fixed.A=fixed.A),
      silent = TRUE
    )

    if (!inherits(fit_b, "try-error")) {
      if(is.null(fixed.A)==TRUE){
        A_b <- fit_b$par[1]
        k_b <- fit_b$par[2]}else{
          A_b <- fixed.A
          k_b <- fit_b$par[1]
        }
      boot_params[b, ] <- c(A_b, k_b)
      boot_preds[b, ] <- size.at.age(A_b, k_b, ages, size0,model=model)
    }
  }

  pred_lower <- apply(boot_preds, 2, quantile, probs = 0.025, na.rm = TRUE)
  pred_upper <- apply(boot_preds, 2, quantile, probs = 0.975, na.rm = TRUE)

  ci <- cbind.data.frame(ages=ages,upper=pred_upper,lower=pred_lower)

  ci_params <- data.frame(
    param = c("A", "k"),
    est   = apply(boot_params, 2, median, na.rm = TRUE),
    lower = apply(boot_params, 2, quantile, probs = 0.025, na.rm = TRUE),
    upper = apply(boot_params, 2, quantile, probs = 0.975, na.rm = TRUE),
    n_successful = apply(boot_params, 2, function(x) sum(!is.na(x)))
  )

  list(size.ci = ci,params.ci = ci_params)

}

#' @noRd
bootstrap.ml <- function(nboot, size1, size2, td, start.vals, ages, size0, model, seed=1701,fixed.A,optimizer="L-BFGS-B") {
  set.seed(seed)
  n <- length(size1)

  if (is.null(fixed.A)) {
    ssq.lower <- c(1e-6, 1e-6)
    ml.lower  <- c(1e-6, 1e-6, 1e-6)
  } else {
    ssq.lower <- c(1e-6)
    ml.lower  <- c(1e-6, 1e-6)
  }


  boot_params <- matrix(NA, nrow = nboot, ncol = 3,          # 3 not 2 — now includes sigma
                        dimnames = list(NULL, c("A", "k", "sigma")))
  boot_preds <- matrix(NA, nrow = nboot, ncol = length(ages))

  for (b in 1:nboot) {
    idx    <- sample(1:n, replace = TRUE)
    size1_b <- size1[idx]; size2_b <- size2[idx]; td_b <- td[idx]

    # --- Stage 1: SSQ for robust starting values ---

    if(is.null(fixed.A)==TRUE){
      fit_ssq <- try(
        optim(start.vals[c("A","k")], ssq,  size1 = size1_b, size2 = size2_b, td = td_b,  method = optimizer, lower = ssq.lower,model=model,fixed.A=fixed.A),
        silent = TRUE)}else{
          fit_ssq <- try(
            optim(start.vals[c("k")], ssq,  size1 = size1_b, size2 = size2_b, td = td_b,  method = optimizer, lower = ssq.lower,model=model,fixed.A=fixed.A),
            silent = TRUE)}

    if (inherits(fit_ssq, "try-error")) next   # skip to next bootstrap replicate

    ml.start <- if(is.null(fixed.A)==TRUE){c(fit_ssq$par[1],fit_ssq$par[2],sigma=sqrt(fit_ssq$value/n))}else{
      c(fit_ssq$par[1],sigma=sqrt(fit_ssq$value/n)) }

    # --- Stage 2: ML fit ---
    fit_ml <- try(
      optim(ml.start, ml,
            size1 = size1_b, size2 = size2_b, td = td_b,
            method = optimizer, lower = ml.lower,model=model,fixed.A=fixed.A),
      silent = TRUE)

    if (inherits(fit_ml, "try-error")) next


    if(is.null(fixed.A)==TRUE){
      A_b <-fit_ml$par[1]
      k_b     <- fit_ml$par[2]
      sigma_b <- fit_ml$par[3]}else{
        A_b <- fixed.A
        k_b     <- fit_ml$par[1]
        sigma_b <- fit_ml$par[2]}

    boot_params[b, ] <- c(A_b, k_b, sigma_b)
    boot_preds[b, ]  <- size.at.age(A_b, k_b, ages, size0,model=model)
  }

  pred_lower <- apply(boot_preds, 2, quantile, probs = 0.025, na.rm = TRUE)
  pred_upper <- apply(boot_preds, 2, quantile, probs = 0.975, na.rm = TRUE)

  ci <- cbind.data.frame(ages = ages, upper = pred_upper, lower = pred_lower)

  ci_params <- data.frame(
    param = c("A", "k", "sigma"),
    est   = apply(boot_params, 2, median,   na.rm = TRUE),
    lower = apply(boot_params, 2, quantile, probs = 0.025, na.rm = TRUE),
    upper = apply(boot_params, 2, quantile, probs = 0.975, na.rm = TRUE),
    n_successful = apply(boot_params, 2, function(x) sum(!is.na(x)))
  )

  list(size.ci = ci,params.ci = ci_params)
}

---
title: "MRgrowth: Growth Models for Mark-Recapture Data"
author: "Donald T. McKnight"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with MRgrowth}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview

'MRgrowth' provides tools for estimating body-size growth rates from mark-recapture data. It is intended for situations where individuals do not have known ages, a common scenario in field studies of reptiles and other long-lived animals, and it has been designed to be as simple and easy to use as possible. 

'MRgrowth' supports three growth models:

- **von Bertalanffy** (`"vb"`; using the Faben's reformulation)
- **Gompertz** (`"gompertz"`)
- **Logistic** (`"logistic"`)

This vignette walks through a complete workflow using a turtle mark-recapture dataset.

---


## Example data

'MRgrowth' includes a built-in example data set `MRdata`, which contains mark-recapture data for a turtle population. Each row represents a single capture event for an individual.

```{r data}

library(MRgrowth)

data(MRdata)
head(MRdata)
str(MRdata)
```

The data set contains three columns:

- `id` — a unique identifier for each individual
- `date` — the date of the capture event (in Date format; POSIX formats with associated times are also supported)
- `size` — the body size of the individual at capture (the example is in mm, but any units are acceptable)

Some individuals were captured more than twice, and there are some missing values. 'MRgrowth' handles both situations through the `format.data()` function described below.

---

## Step 1: Format the data

Raw mark-recapture data are typically in long format (one row per capture event), but 'MRgrowth' requires a wide format (one row per pair of capture events). The `format.data()` function handles this conversion. Filtering for NAs and individuals that were only captured once is handled internally and you do not need to filter the data beforehand (see below).

```{r format}
formatted.data <- format.data(
  x        = MRdata,
  id.col   = "id",
  date.col = "date",
  size.col = "size",
  units    = "years",
  retain   = "max"
)

head(formatted.data)
```

The output contains one row per pair of capture events, with columns for:

- `id` — individual identifier
- `size1` — size at first capture
- `size2` — size at recapture
- `date1` / `date2` — dates of each capture
- `td` — time elapsed between captures in the units you specified

Only the size1, size2, and td columns are needed for subsequent functions. The date1 and date2 columns are provided in case users need to match this output with other aspects of their data.

**Note:** Sizes can be in any units you like, and those units will be carried throughout all subsequent functions. Thus, if you enter your data in mm, all results are for mm. Likewise, whatever time unit you set in `format.data()` will be the time unit for all subsequent functions and results.

### Handling multiple recaptures

The `retain` argument controls what happens when an individual was captured more than twice. As a general rule, individuals should only be included once to avoid pseudoreplication unless you are running a model with an error structure that handles replication per individual (those models are not currently supported in 'MRgrowth')

- `"max"` (default) — uses the first and last capture, maximizing the time interval and therefore the information available for estimating growth. This is recommended in most situations.
- `"random"` — randomly selects two captures. Use the `seed` argument for reproducibility.
- `"all"` — retains all consecutive pairs of captures (e.g., captures 1&2, 2&3, 3&4). Note that this introduces **pseudoreplication** because measurements from the same individual are not independent, and should be used with caution.

### NAs and data filtering

Any capture events with missing data are removed before the `retain` argument is applied. This means that if an individual was captured three times but had missing size data at the first capture, the function will use the second and third captures even if `retain = "max"`.

Additional, any individuals that were only measured once are filtered out of the data set. 

---

## Step 2: Fit growth models

The `calc.growth()` function is the primary function in 'MRgrowth'. It fits one or more growth models to the formatted data, returns the fitted values of asymptotic size (A) and growth rate (k), calculates bootstrapped confidence intervals, and optionally produces a plot of growth curves.

### Key arguments

**`size0`** — the body size at birth or hatching. This does not affect the estimates of A or k, but it does affect the predicted sizes at each age in `age.vect`. Use a species-typical birth or hatchling size. If you do not provide a value, the function will use half the smallest observed size, which may be unrealistic.

**`age.vect`** — a vector of ages for which predicted sizes and confidence intervals will be calculated. Choose a range that covers the full lifespan of your species. If not provided, the function defaults to ages 1 through 4 times the largest observed time interval, which may not cover the full age range. The number of values in this vector has very little impact on run time, so it is best to use fine-scale intervals (e.g., every year) to produce smooth curves.

**`models`** — a vector of models to run and evaluate (any combination of "vb","logistic","gompertz"). The default is to run and compare all three.

**`runs`** — 95% confidence intervals are calculated by bootstrapping, and the runs argument determines the number of bootstrap replicates (higher values = better estimates but longer run times). The default of 5000 is generally appropriate for publication. Lower values (e.g., 100) can be used during exploratory analysis to reduce run time. This argument only affects the confidence intervals, not the the actual estimates of A, k, or model diagnostics (e.g., (AIC). Therefore, for large data sets, you may want to do an initial run with bootstrapping essentially turned off by setting it to 1. Use that run to determine which model is the best fit, then run a second analysis using a high number of runs (e.g., 5000) and only the best fit model. Note that the seed argument is passed internally to `set.seed()`, so results are repeatable.

**`type`** — by default, 'MRgrowth' uses maximum likelihood ("ML") estimation, but it can be changed to least squares ("OLS"). Usually, ML and OLS will produce identical results; however, OLS will not return AIC values. Occasionally, ML may result in convergence issues which are not present in OLS. Therefore, if an ML model does not converge, you may want to try it with OLS.

**`fixed.A`** — in a limited number of cases, you may want to evaluate only the k parameter and supply a fixed asymptotic value (A) based on existing knowledge (e.g., a different study of a different population). As a general rule, this is not advisable, but can be used in some cases such as small sample sizes. Interpreting comparisons of models with and without a fixed A value is complex. AICs are inherently penalized by the number of parameters involved; thus, models with fixed.A have fewer parameters, which will result in lower AICs unless the fit is substantially worse. As a result, if you run models with and without a fixed A, and the fixed values are close to the values that would have been estimated, you will get lower AICs even if the other parameters (k) do not actually fit as well. This is simply because you had one fewer parameter penalizing the AIC score. Further, for those AICs to be valid, the value of A must have been derived entirely independently (e.g., a different population). If it was based on a previous run or some aspect of the population being studied (e.g., max observed body size) then it is not independent and the AIC score has not been accurately penalized for the number of parameters. To help with this, two sets of AIC scores are returned when a value is supplied for fixed.A. The "fixed" scores were calculated based on two parameters being estimated (k and sigma). These are the "correct" AICs for models with a fixed A, but they come with the caveats described above. The "AIC" and "AICc" columns contain the AICs based on three parameters being estimated (A, k, and sigma), and may be more appropriate if the selection of A was in anyway biased. Other model diagnostics like convergence issues and NLL should also be consulted. 

**`other arguments`** — The other inputs are set to reasonable defaults and, in most cases, they do not need to be altered. Indeed, in most cases, only the data set, size0, and age.vect need to be supplied for a useful run (assuming your data is the output of `format.data()`).

Adjusting initial.A and initial.k can result in slight improvements in run time, but they are generally not noticeable. In concept, initial.A and initial.k values could affect convergence in the ML models, but to avoid this `calc.growth()` internally uses the initial values in an OLS model to produce suitable inputs for an ML model, and the final results are based on the ML model. So, in practice, initial values have no effect on the final result.

---

### Basic usage

Let's run a simple comparison of all three models, looking at the expected size at ages 1-50 (years), with a hatchling size of 50 mm. For this example, we will keep runs low (100) to reduce run time, but for actual analyses, you want several thousand runs. Most values will be left on their default settings.

```{r fit_fast}
# Using low runs for vignette build speed
results <- calc.growth(
  x           = formatted.data,
  size0       = 50,
  age.vect    = 1:50,
  models      = c("vb", "gompertz", "logistic"),
  type        = "ML",
  runs        = 100,
  return.plot = TRUE
)
```

## Step 3: Examine the output

`calc.growth()` returns a list with two elements.

### Parameters

```{r params}
results$parameters
```

Each row contains the fitted parameters and diagnostics for one model:

- **`A`** — the estimated asymptotic size. Note that A is an average asymptotic size for the population. Thus, you will probably have some individuals larger than A.
- **`k`** — the growth rate coefficient. Higher values indicate faster growth toward the asymptote.
- **`A.lower` / `A.upper`** and **`k.lower` / `k.upper`** — the 95% bootstrapped confidence intervals for each parameter.
- **`convergence`** — a code from `optim()`. A value of `0` indicates the optimizer converged successfully. Any other value indicates a convergence issue and the results should be treated cautiously.
- **`NLL`** — the negative log-likelihood of the fitted model (only returned if type = "ML" [default]).
- **`SS`** — the sum of squares of the fitted model (only returned if type = "SS").
- **`AIC`** / **`AICc`** — information criteria for model comparison (only returned if type = "ML" [default]; see the fixed.A section in step 2).

### Sizes at ages

```{r sizes}
head(results$sizes.at.ages)
```

This data frame contains the predicted size at each age in `age.vect` for each model, along with the upper and lower bootstrapped 95% confidence intervals. These are the data behind the growth curve plot and are available for more refined plotting in ggplot2 or other graphing software.

---

## Step 4: Compare models

When multiple models are fitted, the AIC and AICc values in `results$parameters` can be used to compare them. Lower values indicate a better fit relative to model complexity.

NLL or SS values can also be useful (lower values = better fits).

Make sure to check the convergence column. Values other than 0 indicate convergence issues and generally should not be trusted even if they have low AICs.

In this case, no models had convergence issues and the 'vb' (von Bertalanffy) appears to be the best fit, so we will use it in all subsequent steps.

```{r}
#look at model diagnostics
print(results$parameters)


# Extract parameters from the best-fitting model
best.model <- results$parameters[which(results$parameters$model=="vb"), ]

```
Additional checks can be performed by using the resulting model parameters and the `age.at.size()` and `size.at.age()` functions (see Step 5) 

---

## Step 5: Additional utility functions

Once you have determined the best fit model, it is a good idea to interrogate it further to see how well it actually fits the data. Simply being the "best" model out of the three we tried does not mean that the model is actually a good fit (it may simply be not as bad as the others). This section first walks introduces some aditional functions, then explains how to use them to interrogate your model.

### Predicted size at a given age

Once you have estimates of A and k, you can use `size.at.age()` to predict the size of an individual at any age.

```{r size_at_age}
#calculate expected sizes for a 10, 20 and 30 year old turtle
#note that the model argument specifies that we are using a vb model

size.at.age(
  A     = best.model$A,
  k     = best.model$k,
  age   = c(10, 20, 30),
  size0 = 50,
  model = "vb")

```

This will give the same results as the $sizes.at.ages output from `calc.growth()`, but you can use it to examine different sizes, different birth/hatching sizes, or slight variations in A and k.  

### Predicted age at a given size

Once you have estimates of A and k, you can use `age.at.size()` to estimate the age of an individual based on its size.

```{r age_at_size}

#Estimate the age (years) of a 150, 250, and 350 mm turtle
#note that the model argument specifies that we are using a vb model

age.at.size(
  A     = best.model$A,
  k     = best.model$k,
  size  = c(150, 250, 350),
  size0 = 50,
  model = "vb")

```

Note that ages cannot be calculated for sizes greater than A; these will return `NaN`.

### Predicted size after a time interval

The three growth functions (`vb()`, `gompertz()`, `logistic()`) can also be used directly to predict the size of an individual after a given time interval, starting from a known size. Note again that all units (including time) should be the same as the initial units used in `format.data()`.

```{r growth_funcs}
# Predict size of a 200 mm turtle after 3 years using the vb model
#note that unlike most functions in 'MRgrowth', each model as a separate function, rather than specifying the model in a single function

vb(
  A     = best.model$A,
  k     = best.model$k,
  size1 = 200,
  td    = 3
)
```

### Further model interrogations

You can use these functions to perform further checks on your model.

#### Comparing observed and predicted sizes at recapture

We can use the size1 and time difference in our formatted input data to predict what size turtles should be at recapture. We can then compare those sizes to the size2 data to see how well the model predicted them.

```{r}

predicted.size2 <- vb(
  A     = best.model$A,
  k     = best.model$k,
  size1 = formatted.data$size1, #use size 1 data
  td    = formatted.data$td #use time difference in the data
)

plot(x = formatted.data$size2,
     y = predicted.size2,
     xlab = "Observed size2",
     ylab = "Estimated size2",
     pch = 16,
     col = "blue",
     bg  = "white",
     panel.first = rect(par("usr")[1], par("usr")[3],
                        par("usr")[2], par("usr")[4],
                        col = "white"))
abline(lm(predicted.size2 ~ formatted.data$size2),
       col = "black",
       lwd = 1)
r2 <- summary(lm(predicted.size2 ~ formatted.data$size2))$r.squared
legend("topleft",
       legend = bquote(R^2 == .(round(r2, 4))),
       bty    = "n")

```

Here you can see that the values match up pretty well with a high R2. So the model did a good job. If the R2 was low, it would indicate that even the "best" model is doing a poor job of fitting this data and should be treated with skepticism.

Another important check is the spacing of the residuals along the line. If, for example, there were large deviations only on the left (small side), it would suggest that the model is doing a good job of estimating growth of large individuals, but struggling with small individuals.

**Note:** Because of the models' math, for `vb()`, `gompertz()`, and `logistic()` any time that size1 is < A, the estimated size will be less than size 1. Therefore, you may want to either filter those out (the best option for R2 values) or replace them with size1 in the output.

```{r}
#remove any individuals where size1 > A
observed.size2.b <- formatted.data$size2[which(formatted.data$size1 < best.model$A)]
predicted.size2.b <- predicted.size2[which(formatted.data$size1 < best.model$A)]

plot(x = observed.size2.b,
     y = predicted.size2.b,
     xlab = "Observed size2",
     ylab = "Estimated size2",
     pch = 16,
     col = "blue",
     bg  = "white",
     panel.first = rect(par("usr")[1], par("usr")[3],
                        par("usr")[2], par("usr")[4],
                        col = "white"))
abline(lm(predicted.size2.b ~ observed.size2.b),
       col = "black",
       lwd = 1)
r2 <- summary(lm(predicted.size2.b ~ observed.size2.b))$r.squared
legend("topleft",
       legend = bquote(R^2 == .(round(r2, 4))),
       bty    = "n")

```

##### Comparing observed and predicted sizes and ages for individuals with known-age data

If you have a subset of individuals with known ages, you can use the `size.at.age()` and `age.at.size()` functions to further check the models.

Let's say, for example, that in addition to the animals used to generate the models, we have 4 individuals of know ages. They are 5, 10, 12, and 20 years old, and are 200, 320, 330, and 400 mm. Let's start by using their ages to predict their sizes. We'll convert the results into percent differences to make them easier to interpret.

```{r}
predicted.size <- size.at.age(A=best.model$A,
                              k=best.model$k,
                              age=c(5,10,12,20),
                              size0=50,
                              model="vb")

#expected sizes given their ages
print(predicted.size)

#percent difference between observed and predicted sizes
print((c(200,320,330,400)-predicted.size)/c(200,320,330,400)*100)
```

Mostly, those size estimates line up pretty well. As a point of comparison, let's do the same but with the logistic model, which did not fit as well.

```{r}
logistic.model <- results$parameters[which(results$parameters$model=="logistic"), ]

predicted.size.logistic <- size.at.age(A=logistic.model$A,
                              k=logistic.model$k,
                              age=c(5,10,12,20),
                              size0=50,
                              model="logistic")

#predicted sizes based on the logistic model
print(predicted.size.logistic)

#percent difference between observed and predicted sizes for the logistic model
print((c(200,320,330,400)-predicted.size.logistic)/c(200,320,330,400)*100)

```

These estimates are much further off, especially for smaller individuals, which makes sense, given that the logistic model did not fit as well.

Now we can invert things and look at the expected ages for those sizes.

```{r}
predicted.age <- age.at.size(A=best.model$A,
                              k=best.model$k,
                              size=c(200,320,330,400),
                              size0=50,
                              model="vb")

print(predicted.age)
```

Again, not too bad, suggesting that that the vb model is doing a good job.

---

## References

Edmonds, D., et al. (2021). Growth and population structure of a long-lived reptile.
*PLOS ONE*. <doi:10.1371/journal.pone.0259978>

von Bertalanffy, L. (1957). Quantitative laws in metabolism and growth.
*The Quarterly Review of Biology*, 32, 217--231.

Fabens, A.J. (1965). Properties and fitting of the von Bertalanffy growth curve.
*Growth*, 29, 265--289.

Gompertz, B. (1833). On the nature of the function expressive of the law of human
mortality, and on a new mode of determining the value of life contingencies.
*Philosophical Transactions of the Royal Society of London*, 123, 252--253.

Schoener, T.W. and Schoener, A. (1978). Estimating and interpreting body-size
growth in some Anolis lizards. *Copeia*, 1978, 390--405.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/age.at.size.R
\name{age.at.size}
\alias{age.at.size}
\title{Age at size}
\usage{
age.at.size(A, k, size, size0, model)
}
\arguments{
\item{A}{(numeric) Asymptotic body size (can be calculated in \code{calc.growth()})}

\item{k}{(numeric) Growth rate (can be calculated in \code{calc.growth()})}

\item{size}{(numeric vector) The size(s) for which you want to know the age (should be in the same units used to calculate A and k)}

\item{size0}{(numeric) Initial body size at birth/hatching (should be in the same units used to calculate A and k). This will generally be a species average, rather than being specific per individual, but it can be a vector of specific birth/hatching sizes per individual if known.}

\item{model}{(character) The model used to calculate A and k. One of "vb", "gompertz","logistic".}
}
\value{
Vector of expected age(s)
}
\description{
Calculate expected age at a given size
}
\details{
It takes a vector of sizes, initial size for the species, and A and k from \code{calc.growth()} and returns the expected age at a given size based on the specified model.

\strong{Note}: Age cannot be calculated for sizes >A, resulting in NaN.
}
\examples{
#For this example, assume size is in mm and time is in years
#Use the gompertz formula to calculate the expected age of a 350 mm
#turtle for a species that hatches at an average size of 50 mm
age.at.size(A = 396.6738,
 k = 0.2029850,
 size = 350,
 size0 = 50,
 model="gompertz")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/size.at.age.R
\name{size.at.age}
\alias{size.at.age}
\title{Size at age}
\usage{
size.at.age(A, k, age, size0, model)
}
\arguments{
\item{A}{(numeric) Asymptotic body size (can be calculated in \code{calc.growth()})}

\item{k}{(numeric) Growth rate (can be calculated in \code{calc.growth()})}

\item{age}{(numeric vector) The age(s) for which you want to know the size (should be in the same units used to calculate A and k)}

\item{size0}{(numeric) Initial body size at birth/hatching (should be in the same units used to calculate A and k). This will generally be a species average, rather than being specific per individual, but it can be a vector of specific birth/hatching sizes per individual if known.}

\item{model}{(character) The model used to calculate A and k. One of "vb", "gompertz","logistic".}
}
\value{
Vector of expected size(s)
}
\description{
Calculate expected size at a given age
}
\details{
It takes a vector of ages, initial size for the species, and A and k from \code{calc.growth()} and returns the expected body sizes at a given age based on the specified model.
}
\examples{
#For this example, assume size is in mm and time is in years
#Use the gompertz formula to calculate the expected size of a 5-year old
#turtle for a species that hatches at an average size of 50 mm
size.at.age(A = 396.6738,
 k = 0.2029850,
 age = 5,
 size0 = 50,
 model="gompertz")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc.growth.R
\name{calc.growth}
\alias{calc.growth}
\title{Calculate growth curves}
\usage{
calc.growth(
  x,
  size0 = NULL,
  age.vect = NULL,
  models = c("vb", "gompertz", "logistic"),
  fixed.A = NULL,
  initial.A = NULL,
  initial.k = 0.5,
  size1.col = "size1",
  size2.col = "size2",
  times.col = "td",
  type = "ML",
  runs = 5000,
  seed = 1701,
  return.plot = TRUE,
  optimizer = "L-BFGS-B"
)
}
\arguments{
\item{x}{(data frame) Output from \code{MRgrowth::format.data()} Data frame with columns of size at capture, size at recapture, and amount of time between captures.}

\item{size0}{(numeric) Initial body size at birth/hatching (should be in the same units used to calculate A and k). This will generally be a species average, rather than being specific per individual, but it can be a vector of specific birth/hatching sizes per individual if known. Note that this will not affect the final values of A, k, or their confidence intervals, but it will affect the estimated sizes at ages.}

\item{age.vect}{(numeric vector) Vector of ages for which you would like expected sizes and confidence intervals around the size (used for plotting growth curves). If age.vect is not supplied, a vector from 1 to 4x the largest time interval will be used (default = NULL)}

\item{models}{(character vector) The model(s) used to calculate A and k. Can be any combination of "vb", "gompertz","logistic". Fitted parameters, model diagnostics, and sizes at expected ages will be returned for all included models (default = c("vb", "gompertz","logistic"))}

\item{fixed.A}{(numeric) This fixes the asymptotic size (A) at the specified value, meaning that only the growth rate (k) will be estimated. Use with caution (see vignette; default = NULL)}

\item{initial.A}{(numeric) Starting value for the search algorithm for the asymptotic size (A). This has no impact on the results, but an initial.A that is close to the final A will result in a faster run time (default = max size in your data set).}

\item{initial.k}{(numeric) Starting value for the search algorithm for the growth rate (k). This has no impact on the results, but an initial.k that is close to the final k will result in a faster run time (default = 0.5).}

\item{size1.col}{(character) Name of column containing size data at capture (default = "size1"; the output from \code{MRgrowth::format.data()})}

\item{size2.col}{(character) Name of column containing size data at recapture (default = "size2"; the output from \code{MRgrowth::format.data()})}

\item{times.col}{(character) Name of column containing time interval between captures (default = "td"; the output from \code{MRgrowth::format.data()})}

\item{type}{(character) Method used of fitting the model. One of "ML" (Maximum Likelihood) or "OLS" (Ordinary Least Squares; default = "ML")}

\item{runs}{(integer) The number of bootstraps to use for calculating confidence intervals. Large values result in long run times (see Vignette; default = 5000)}

\item{seed}{(integer) Seed to use for random selections when calculating bootstrapped confidence intervals (default = 1701).}

\item{return.plot}{(logical) Determines whether a plot of growth curves should be printed (default = TRUE)}

\item{optimizer}{(character) One of the optimization methods available in \code{\link[stats]{optim}} (default = "L-BFGS-B")}
}
\value{
A list of two objects:
\describe{
 \item{parameters}{A data frame with one row per model containing the following columns:
    \describe{
      \item{model}{The growth model fitted}
      \item{A}{Estimated asymptotic size}
      \item{A.lower}{Lower 95\% bootstrap confidence interval for A}
      \item{A.upper}{Upper 95\% bootstrap confidence interval for A}
      \item{k}{Estimated growth coefficient}
      \item{k.lower}{Lower 95\% bootstrap confidence interval for k}
      \item{k.upper}{Upper 95\% bootstrap confidence interval for k}
      \item{convergence}{Convergence code from \code{\link[stats]{optim}}; 0 indicates successful convergence}
      \item{NLL}{Negative log-likelihood of the fitted model (only returned when \code{type="ML"})}
      \item{SS}{Sum of squares (only returned when \code{type="OLS"})}
      \item{AIC}{Akaike Information Criterion (only returned when \code{type="ML"})}
      \item{AICc}{Sample size corrected AIC (only returned when \code{type="ML"})}
      \item{fixed.AIC}{Akaike Information Criterion calculated with 2 parameters when A is fixed (only returned when \code{type="ML"} and a value is entered for \code{fixed.A})}
      \item{fixed.AICc}{Sample size corrected AIC calculated with 2 parameters when A is fixed (only returned when \code{type="ML"} and a value is entered for \code{fixed.A})}

    }
  }
  \item{sizes.at.ages}{A data frame with one row per age per model containing the following columns:
    \describe{
      \item{model}{The growth model fitted}
      \item{age}{Age from \code{age.vect}}
      \item{size}{Predicted size at a given age}
      \item{upper}{Upper 95\% bootstrap confidence interval for predicted size}
      \item{lower}{Lower 95\% bootstrap confidence interval for predicted size}
    }
  }
}
}
\description{
Calculate, compare, and plot growth curves based on different models
}
\details{
This is the primary function for MRgrowth. It takes the output from \code{format.data()}, calculates growth curves the selected model(s), and returns the model parameters and the data for plotting curves. The output includes model diagnostics like AIC values that can be used to evaluate model fit and compare models.
}
\examples{

data(MRdata)

#format the data
formatted.data <- format.data(
 x = MRdata,
 id.col = "id",
 date.col = "date",
 size.col = "size",
 units = "years",
 retain = "max")

#get growth curves for the gompertz and logistic methods.
#runs are set to 10 to keep the run time low.
#For actual CI estimates, much higher values are needed
#likewise, the plot function has been turned off
g.result <- calc.growth(formatted.data,
   size0=50,
   age.vect=c(1:40),
   models=c("gompertz","logistic"),
   runs=10,
   return.plot=FALSE)

}
\references{
von Bertalanffy, L. (1957). Quantitative laws in metabolism and growth.
\emph{The Quarterly Review of Biology}, 32, 217-231.

Fabens, A.J. (1965). Properties and fitting of the Von Bertalanffy growth curve.
\emph{Growth}, 29, 265-289.

Gompertz, B. (1833). On the nature of the function expressive of the law of human
mortality, and on a new mode of determining the value of life contingencies.
\emph{Philosophical Transactions of the Royal Society of London}, 123, 252-253.

Schoener, T.W. and Schoener, A. (1978). Estimating and interpreting body-size
growth in some Anolis lizards. \emph{Copeia}, 1978, 390-405.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vb.R
\name{vb}
\alias{vb}
\title{von Bertalanffy growth}
\usage{
vb(A, k, size1, td)
}
\arguments{
\item{A}{(numeric) Asymptotic body size (can be calculated in \code{calc.growth()})}

\item{k}{(numeric) Growth rate (can be calculated in \code{calc.growth()})}

\item{size1}{(numeric vector) Initial body size (should be in the same units used to calculate A and k)}

\item{td}{(numeric vector) Time since \code{size1} (should be in the same units used to calculate A and k)}
}
\value{
Vector of expected size(s) after the time interval(s) specified in \code{td}
}
\description{
Calculate size after a given time interval using the Faben's reformulation of the von Bertalanffy formula
}
\details{
It takes a vector of initial sizes, vector of amounts of time since those sizes were measured, and A and k from \code{calc.growth(model="vb")} and returns the expected body sizes after the specified time intervals.

\strong{Note}: If size1 > A, the resulting size estimate will be less than size1
}
\examples{
#For this example, assume size is in mm and time is in years
#calculate expected sizes of individuals that are 200 and 300 mm after 2 and 1.5 years, respectively
vb(A = 403.7077,
 k = 0.1365507,
 size1 = c(200,300),
 td = c(2,1.5))

}
\references{
von Bertalanffy, L. (1957). Quantitative laws in metabolism and growth.
\emph{The Quarterly Review of Biology}, 32, 217-231.

Fabens, A.J. (1965). Properties and fitting of the von Bertalanffy growth curve.
\emph{Growth}, 29, 265-289.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.data.R
\name{format.data}
\alias{format.data}
\title{Format data}
\usage{
\method{format}{data}(
  x,
  id.col = "id",
  date.col = "date",
  size.col = "size",
  units = "years",
  retain = "max",
  seed = 1701
)
}
\arguments{
\item{x}{(data frame) Input data containing one row per capture event and columns for individual ID, date/time (formatted as a Date or POSIX), and body size measurement.}

\item{id.col}{(character) Name of column containing unique individual IDs (default = "id")}

\item{date.col}{(character) Name of column containing dates/times (default = "date")}

\item{size.col}{(character) Name of column containing body size measurements (default = "size")}

\item{units}{(character) Units to be returned for time intervals between measurements. One of "hours", "days", "weeks", "years" (default = "years")}

\item{retain}{(character) Determines how the function should handle more than two measurements per individual. "max" returns the first and last measurement (after removing NAs), "random" = randomly selects two measurements (after removing NAs), "all" = keeps all measurements with growth intervals between measurements 1 and 2, 2 and 3, 3 and 4, etc. Note that "all" results in pseudoreplication. (default = "max")}

\item{seed}{(integer) Seed to use for random selections if \code{retain="random"} (default = 1701).}
}
\value{
Data frame of measurements and capture intervals formatted for MRgrowth functions. Each row is a pair of capture events and associated measurements for an individual.

Column \code{id} = IDs as entered in the input data

Column \code{size1} = size at capture

Column \code{size2} = size at recapture

Column \code{date1} = date at capture

Column \code{date2} = date at recapture

Column \code{td} = time difference between date1 and date2 in the units specified by the units argument.
}
\description{
Format data for other MRgrowth functions
}
\details{
It takes a long-format data frame of capture events and measurements (one row per individual per capture) and converts it into the wide-format (one row per pair of capture events per individual) needed for MRgrowth

If \code{retain} is set to "random", only 1 pair of capture events will be retained per individual. The \code{seed} argument ensures repeatability of the code.

\strong{Note}: Any NA data are removed as part of the function before applying the \code{retain} argument. Thus, if an individual was captured 3 times and only measured the 2nd and 3rd time, the data for the 2nd and 3rd capture events will be returned even if \code{retain="max"}

\strong{Note}: Only individuals with at least 2 capture events (with no NAs) are returned. Any individuals that were never recaptured or did not have all data recorded for at least two recaptures are removed.
}
\examples{

data(MRdata)

#format data with years as the time units and the first and last
#measurement when more than 2 captures are present for an individual.
formatted.data <- format.data(
  x = MRdata,
  id.col = "id",
  date.col = "date",
  size.col = "size",
  units = "years",
  retain = "max"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MRdata.R
\docType{data}
\name{MRdata}
\alias{MRdata}
\title{Example mark-recapture data set}
\format{
A data frame with 43 rows and 3 variables:
\describe{
  \item{id}{Unique ID for each individual}
  \item{size}{Individual body size at each capture}
  \item{date}{Date of each measrement in Date format}
}
}
\source{
Hypothetical data over 2-3 capture events
}
\usage{
data(MRdata)
}
\description{
Example mark-recapture data set of male and female turtles from two populations
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gompertz.R
\name{gompertz}
\alias{gompertz}
\title{Gompertz growth}
\usage{
gompertz(A, k, size1, td)
}
\arguments{
\item{A}{(numeric) Asymptotic body size (can be calculated in \code{calc.growth()})}

\item{k}{(numeric) Growth rate (can be calculated in \code{calc.growth()})}

\item{size1}{(numeric vector) Initial body size (should be in the same units used to calculate A and k)}

\item{td}{(numeric vector) Time since \code{size1} (should be in the same units used to calculate A and k)}
}
\value{
Vector of expected size(s) after the time interval(s) specified in \code{td}
}
\description{
Calculate size after a given time interval using Gompertz formula
}
\details{
It takes a vector of initial sizes, vector of amounts of time since those sizes were measured, and A and k from \code{calc.growth(model="gompertz")} and returns the expected body sizes after the specified time intervals.

\strong{Note}: If size1 > A, the resulting size estimate will be less than size1
}
\examples{
#For this example, assume size is in mm and time is in years
#calculate expected sizes of individuals that are 200 and 300 mm after 2 and 1.5 years, respectively
gompertz(A = 396.6738,
 k = 0.2029850,
 size1 = c(200,300),
 td = c(2,1.5))

}
\references{
Gompertz, B. (1833). On the nature of the function expressive of the law of human
mortality, and on a new mode of determining the value of life contingencies.
\emph{Philosophical Transactions of the Royal Society of London}, 123, 252-253.
}
